const formatDateTime = (date) => {
  if (!date) return
  const dateObj = new Date(date)
  const year = dateObj.getFullYear()
  const month = String(dateObj.getMonth() + 1).padStart(2, '0')
  const day = String(dateObj.getDate()).padStart(2, '0')
  const hours = String(dateObj.getHours()).padStart(2, '0')
  const minutes = String(dateObj.getMinutes()).padStart(2, '0')
  const seconds = String(dateObj.getSeconds()).padStart(2, '0')

  return `${year}-${month}-${day} ${hours}:${minutes}:${seconds}`
}

const yieldCPU = () => {
  return new Promise((resolve) => {
    setTimeout(resolve, 0)
  })
}

const initWasm = (url) => {
  importScripts(url)
  Module.onRuntimeInitialized = async function () {
    let last_timestamp = 0
    _Init()
    while (true) {
      _Loop()
      let trustDO = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
      for (let i = 0; i < 10; i++) {
        trustDO[i] = _GetTrustDO(i)
      }
      doCom = trustDO[0] || trustDO[1] || trustDO[2] || trustDO[3] || trustDO[4] || trustDO[5] || trustDO[6] || trustDO[7] || trustDO[8] || trustDO[9]
      for (let i = 0; i < 10; i++) {
        drawPlc('DO', i, trustDO[i])
      }
      for (let i = 0; i < 2; i++) {
        drawPlc('AO', i, Math.floor(_GetTrustAO(i) / 100))
      }
      // drawPlc('time', 0, _GetTimeStamp())
      let now_timestamp = _GetTimeStamp()
      if (now_timestamp != last_timestamp) { // 防止反复高频调用postMessage，把主线程的性能也拉低了。
        postMessage({ type: 'setTime', val: now_timestamp })
        last_timestamp = now_timestamp
      }
      await yieldCPU() // 让出cpu，保证有间隔进入消息接收回调
    }
  }
}

let canvasEl
let ctx

const r = 26 // 圆的半径
const l_x = 47 + r  // 左上角圆的圆的心x坐标
const r_x = 589 + r // 右上角圆的圆的心x坐标
const lr_y = 57 + r // 最上面的圆的圆心y坐标
const lr_dis = 74 // 上下两个圆圆心的距离
// AI与AO的输入框左上角点坐标
const rectX = 346
const rectY = [942, 1016, 1090, 1164, 1286, 1432]

const drawPlc = async (key, index, val, type) => {
  if (!ctx) return
  ctx.font = '30px Noto Sans SC'
  if (type === 'init') {
    const response = await fetch(new URL('/wxapp/canvas-icon/ditu.png', self.location.href).href)
    const blob = await response.blob()
    const imgBitmap = await createImageBitmap(blob)
    ctx.drawImage(imgBitmap, 0, 0)
  }

  switch (key) {
    case 'DI':
    case 'DO':
      // left com
      ctx.beginPath()
      ctx.fillStyle = (DIValMap[0] || DIValMap[1] || DIValMap[2] || DIValMap[3] || DIValMap[4] || DIValMap[5] || DIValMap[6] || DIValMap[7] || DIValMap[8] || DIValMap[9]) ? 'green' : '#D11E27'
      ctx.arc(l_x, lr_y, r, 0, 2 * Math.PI)
      ctx.fill()

      // right com
      ctx.beginPath()
      ctx.fillStyle = doCom ? 'green' : '#D11E27'
      ctx.arc(r_x, lr_y, r, 0, 2 * Math.PI)
      ctx.fill()

      ctx.beginPath()
      ctx.fillStyle = val !== 0 ? 'green' : '#D11E27'
      ctx.arc(key === 'DI' ? l_x : r_x, lr_y + lr_dis * (index + 1), r, 0, 2 * Math.PI)
      ctx.fill()
      break
    case 'AI':
      ctx.fillStyle = '#ffffff'
      ctx.fillRect(rectX + 100, rectY[index] + 6, 100, 38)
      ctx.fillStyle = '#202020'
      ctx.fillText(`${val}`, rectX + 100, rectY[index] + 39)
      break
    case 'AO':
      ctx.fillStyle = '#ffffff'
      ctx.fillRect(rectX + 110, rectY[index + 4] + 6, 95, 38)
      ctx.fillStyle = '#202020'
      ctx.fillText(`${val}`, rectX + 110, rectY[index + 4] + 39)
      break
  }
}

const DIValMap = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
let doCom = false

onmessage = (e) => {
  const { type, url, canvas, position, val, i } = e.data
  if (!canvasEl && type === 'init') {
    canvasEl = canvas
    ctx = canvas.getContext('2d')
    initWasm(url)
    drawPlc(null, null, null, 'init')
    for (let i = 0; i < 10; i++) {
      drawPlc('DI', i, 0, 'draw')
      drawPlc('DO', i, 0, 'draw')
      DIValMap[i] = 0
    }
    for (let i = 0; i < 4; i++) {
      drawPlc('AI', i, 0, 'draw')
    }
    for (let i = 0; i < 2; i++) {
      drawPlc('AO', i, 0, 'draw')
    }
  }
  if (type === 'set') {
    _SetTrustAI(+i, +val * 100)
    drawPlc('AI', +i, +val)
  }
  if (type === 'time') {
    _SetDiffTimeStamp(+val)
  }
  if (type === 'click') {
    if (!position) return
    const { x, y } = position

    // 判断是否点击到DI
    for (let i = 0; i < 10; i++) {
      const p_to_p_dis = Math.sqrt((l_x - x) ** 2 + (lr_y + lr_dis * (i + 1) - y) ** 2)
      if (p_to_p_dis <= r) {
        if (DIValMap[i] === 0) {
          DIValMap[i] = 1
        } else {
          DIValMap[i] = 0
        }
        _SetTrustDI(i, DIValMap[i])
        drawPlc('DI', i, DIValMap[i])
        return
      }
    }
    // 判断是否点击到AI
    for (let i = 0; i < 4; i++) {
      if (x >= rectX && x <= rectX + 208 && y >= rectY[i] && y <= rectY[i] + 58) {
        postMessage({ type: 'AI', i })
        return
      }
    }
  }
}
